@@ -2,11 +2,22 @@
 
 #[cfg(not(target_os = "wasi"))]
 use crate::fs::Mode;
+use crate::fs::Stat;
 #[cfg(not(any(target_os = "netbsd", target_os = "redox", target_os = "wasi")))]
 // not implemented in libc for netbsd yet
 use crate::fs::StatFs;
-use crate::time::Timespec;
+use crate::{io, time::Timespec};
 use io_lifetimes::{AsFd, BorrowedFd};
+#[cfg(all(
+    libc,
+    not(any(
+        target_os = "android",
+        target_os = "linux",
+        target_os = "emscripten",
+        target_os = "l4re",
+    ))
+))]
+use libc::fstat as libc_fstat;
 #[cfg(all(
     libc,
     not(any(
@@ -65,20 +76,12 @@ use libc::off_t as libc_off_t;
         target_os = "l4re"
     )
 ))]
-use libc::{fstatfs64 as libc_fstatfs, lseek64 as libc_lseek};
-#[cfg(all(
-    libc,
-    not(any(target_os = "netbsd", target_os = "redox", target_os = "wasi"))
-))]
-// not implemented in libc for netbsd yet
-use std::mem::MaybeUninit;
-use std::{
-    convert::TryInto,
-    io::{self, SeekFrom},
-};
+use libc::{fstat64 as libc_fstat, fstatfs64 as libc_fstatfs, lseek64 as libc_lseek};
+use std::{convert::TryInto, io::SeekFrom};
 #[cfg(libc)]
 use {
     crate::{negone_err, zero_ok},
+    std::mem::MaybeUninit,
     unsafe_io::os::posish::AsRawFd,
 };
 
@@ -94,14 +97,13 @@ fn _seek(fd: BorrowedFd<'_>, pos: SeekFrom) -> io::Result<u64> {
     let (whence, offset): (libc::c_int, libc_off_t) = match pos {
         SeekFrom::Start(pos) => (
             libc::SEEK_SET,
-            pos.try_into()
-                .map_err(|_convert_err| io::Error::from_raw_os_error(libc::EOVERFLOW))?,
+            pos.try_into().map_err(|_convert_err| io::Error::OVERFLOW)?,
         ),
         SeekFrom::End(offset) => (libc::SEEK_END, offset),
         SeekFrom::Current(offset) => (libc::SEEK_CUR, offset),
     };
     let offset = unsafe { negone_err(libc_lseek(fd.as_raw_fd() as libc::c_int, offset, whence))? };
-    Ok(offset.try_into().unwrap())
+    Ok(offset as u64)
 }
 
 #[cfg(linux_raw)]
@@ -110,14 +112,12 @@ fn _seek(fd: BorrowedFd<'_>, pos: SeekFrom) -> io::Result<u64> {
     let (whence, offset) = match pos {
         SeekFrom::Start(pos) => (
             linux_raw_sys::general::SEEK_SET,
-            pos.try_into().map_err(|_convert_err| {
-                io::Error::from_raw_os_error(linux_raw_sys::errno::EOVERFLOW as i32)
-            })?,
+            pos.try_into().map_err(|_convert_err| io::Error::OVERFLOW)?,
         ),
         SeekFrom::End(offset) => (linux_raw_sys::general::SEEK_END, offset),
         SeekFrom::Current(offset) => (linux_raw_sys::general::SEEK_CUR, offset),
     };
-    crate::linux_raw::lseek(fd, offset, whence)
+    crate::linux_raw::seek(fd, offset, whence)
 }
 
 /// `lseek(fd, 0, SEEK_CUR)`
@@ -131,13 +131,13 @@ pub fn tell<'f, Fd: AsFd<'f>>(fd: Fd) -> io::Result<u64> {
 fn _tell(fd: BorrowedFd<'_>) -> io::Result<u64> {
     let offset =
         unsafe { negone_err(libc_lseek(fd.as_raw_fd() as libc::c_int, 0, libc::SEEK_CUR))? };
-    Ok(offset.try_into().unwrap())
+    Ok(offset as u64)
 }
 
 #[cfg(linux_raw)]
 #[inline]
 fn _tell(fd: BorrowedFd<'_>) -> io::Result<u64> {
-    crate::linux_raw::lseek(fd, 0, linux_raw_sys::general::SEEK_CUR).map(|x| x as u64)
+    crate::linux_raw::seek(fd, 0, linux_raw_sys::general::SEEK_CUR).map(|x| x as u64)
 }
 
 /// `fchmod(fd)`.
@@ -180,6 +180,27 @@ fn _fchmod(fd: BorrowedFd<'_>, mode: Mode) -> io::Result<()> {
     crate::linux_raw::fchmod(fd, mode.bits() as u16)
 }
 
+/// `fstat(fd)`
+#[inline]
+pub fn fstat<'f, Fd: AsFd<'f>>(fd: Fd) -> io::Result<Stat> {
+    let fd = fd.as_fd();
+    _fstat(fd)
+}
+
+#[cfg(libc)]
+fn _fstat(fd: BorrowedFd<'_>) -> io::Result<Stat> {
+    let mut stat = MaybeUninit::<Stat>::uninit();
+    unsafe {
+        zero_ok(libc_fstat(fd.as_raw_fd() as libc::c_int, stat.as_mut_ptr()))?;
+        Ok(stat.assume_init())
+    }
+}
+
+#[cfg(linux_raw)]
+fn _fstat(fd: BorrowedFd<'_>) -> io::Result<Stat> {
+    crate::linux_raw::fstat(fd)
+}
+
 /// `fstatfs(fd)`
 #[cfg(not(any(target_os = "netbsd", target_os = "redox", target_os = "wasi")))] // not implemented in libc for netbsd yet
 #[inline]
@@ -252,31 +273,29 @@ pub fn posix_fallocate<'f, Fd: AsFd<'f>>(fd: Fd, offset: u64, len: u64) -> io::R
 fn _posix_fallocate(fd: BorrowedFd<'_>, offset: u64, len: u64) -> io::Result<()> {
     let offset = offset
         .try_into()
-        .map_err(|_overflow_err| io::Error::from_raw_os_error(libc::EOVERFLOW))?;
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     let len = len
         .try_into()
-        .map_err(|_overflow_err| io::Error::from_raw_os_error(libc::EOVERFLOW))?;
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     let err = unsafe { libc::posix_fallocate(fd.as_raw_fd() as libc::c_int, offset, len) };
 
     // `posix_fallocate` returns its error status rather than using `errno`.
     if err == 0 {
         Ok(())
     } else {
-        Err(io::Error::from_raw_os_error(err))
+        Err(io::Error(err))
     }
 }
 
-#[cfg(any(target_os = "ios", target_os = "macos",))]
+#[cfg(any(target_os = "ios", target_os = "macos"))]
 fn _posix_fallocate(fd: BorrowedFd<'_>, offset: u64, len: u64) -> io::Result<()> {
     let offset: i64 = offset
         .try_into()
-        .map_err(|_overflow_err| io::Error::from_raw_os_error(libc::EOVERFLOW))?;
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     let len = len
         .try_into()
-        .map_err(|_overflow_err| io::Error::from_raw_os_error(libc::EOVERFLOW))?;
-    let new_len = offset.checked_add(len).ok_or_else(|| {
-        io::Error::new(io::ErrorKind::Other, "overflow while allocating file space")
-    })?;
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
+    let new_len = offset.checked_add(len).ok_or_else(|| io::Error::FBIG)?;
     let mut store = libc::fstore_t {
         fst_flags: libc::F_ALLOCATECONTIG,
         fst_posmode: libc::F_PEOFPOSMODE,
@@ -297,12 +316,12 @@ fn _posix_fallocate(fd: BorrowedFd<'_>, offset: u64, len: u64) -> io::Result<()>
 #[cfg(linux_raw)]
 #[inline]
 fn _posix_fallocate(fd: BorrowedFd<'_>, offset: u64, len: u64) -> io::Result<()> {
-    let offset = offset.try_into().map_err(|_overflow_err| {
-        io::Error::from_raw_os_error(linux_raw_sys::errno::EOVERFLOW as i32)
-    })?;
-    let len = len.try_into().map_err(|_overflow_err| {
-        io::Error::from_raw_os_error(linux_raw_sys::errno::EOVERFLOW as i32)
-    })?;
+    let offset = offset
+        .try_into()
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
+    let len = len
+        .try_into()
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     crate::linux_raw::fallocate(fd, 0, offset, len)
 }
 
@@ -310,7 +329,7 @@ fn _posix_fallocate(fd: BorrowedFd<'_>, offset: u64, len: u64) -> io::Result<()>
 /// whether the file descriptor is readable and/or writeable, respectively.
 /// This is only reliable on files; for example, it doesn't reflect whether
 /// sockets have been shut down; for general I/O handle support, use
-/// [`crate::io::is_read_write`].
+/// [`io::is_read_write`].
 #[inline]
 pub fn is_file_read_write<'f, Fd: AsFd<'f>>(fd: Fd) -> io::Result<(bool, bool)> {
     let fd = fd.as_fd();
@@ -340,3 +359,62 @@ pub(crate) fn _is_file_read_write(fd: BorrowedFd<'_>) -> io::Result<(bool, bool)
         _ => unreachable!(),
     }
 }
+
+/// `fsync(fd)`
+#[inline]
+pub fn fsync<'f, Fd: AsFd<'f>>(fd: Fd) -> io::Result<()> {
+    let fd = fd.as_fd();
+    _fsync(fd)
+}
+
+#[cfg(libc)]
+fn _fsync(fd: BorrowedFd<'_>) -> io::Result<()> {
+    unsafe { zero_ok(libc::fsync(fd.as_raw_fd() as libc::c_int)) }
+}
+
+#[cfg(linux_raw)]
+#[inline]
+fn _fsync(fd: BorrowedFd<'_>) -> io::Result<()> {
+    crate::linux_raw::fsync(fd)
+}
+
+/// `fdatasync(fd)`
+#[cfg(not(any(target_os = "ios", target_os = "macos", target_os = "redox")))]
+#[inline]
+pub fn fdatasync<'f, Fd: AsFd<'f>>(fd: Fd) -> io::Result<()> {
+    let fd = fd.as_fd();
+    _fdatasync(fd)
+}
+
+#[cfg(all(
+    libc,
+    not(any(target_os = "ios", target_os = "macos", target_os = "redox"))
+))]
+fn _fdatasync(fd: BorrowedFd<'_>) -> io::Result<()> {
+    unsafe { zero_ok(libc::fdatasync(fd.as_raw_fd() as libc::c_int)) }
+}
+
+#[cfg(linux_raw)]
+#[inline]
+fn _fdatasync(fd: BorrowedFd<'_>) -> io::Result<()> {
+    crate::linux_raw::fdatasync(fd)
+}
+
+/// `ftruncate(fd, length)`
+#[inline]
+pub fn ftruncate<'f, Fd: AsFd<'f>>(fd: Fd, length: u64) -> io::Result<()> {
+    let fd = fd.as_fd();
+    _ftruncate(fd, length)
+}
+
+#[cfg(libc)]
+fn _ftruncate(fd: BorrowedFd<'_>, length: u64) -> io::Result<()> {
+    let length = length.try_into().map_err(|_overflow_err| io::Error::FBIG)?;
+    unsafe { zero_ok(libc::ftruncate(fd.as_raw_fd() as libc::c_int, length)) }
+}
+
+#[cfg(linux_raw)]
+#[inline]
+fn _ftruncate(fd: BorrowedFd<'_>, length: u64) -> io::Result<()> {
+    crate::linux_raw::ftruncate(fd, length)
+}
