@@ -2,11 +2,11 @@
 
 use crate::{
     fs::{AtFlags, Statx},
-    path,
+    io, path,
 };
 use bitflags::bitflags;
 use io_lifetimes::{AsFd, BorrowedFd};
-use std::{ffi::CStr, io};
+use std::ffi::CStr;
 #[cfg(libc)]
 use {crate::zero_ok, std::mem::MaybeUninit};
 
@@ -116,7 +116,7 @@ pub fn statx<'f, P: path::Arg, Fd: AsFd<'f>>(
     mask: StatxFlags,
 ) -> io::Result<Statx> {
     let dirfd = dirfd.as_fd();
-    let path = path.as_c_str()?;
+    let path = path.into_c_str()?;
     _statx(dirfd, &path, flags, mask)
 }
 
